%% Compute the local basis functions and theirs derivatives 
%%  at quadrature vertices   
%% 
%%
%% INPUTS  :  X array 3 x 2 of  coordinates of the triangle 
%%            polyorder : the polynomial order
%%            quadorder : the quadrature order
%%
%% IMPLICIT INPUT: sencond hand function frhs
%%
%% OUTPUTS :  PhQ: array nq x nph of value of basis on quad
%%                 (nq: number of quad points, nph: number of basis function
%%
%%            dxPhQ: array nq x nph of derivatives % first variable of basis on quad
%%
%%            dyPhQ: array nq x nph of derivatives % second variable of basis on quad
%%
%%            weight: array nq x1 of weight in the current triangle
%%
%%            XYQ   : array nq x 2 of coordinate of quad points on the current triangle
%%            (particularly useful when building rhs elementary vector)
%%
%%
%%
%% (c) Jean-Baptiste Apoung Kamga (July 2007)



function [PhQ, dxPhQ, dyPhQ, weight,XYQ] = basis_and_deriv_on_quad(X,polyorder,quadorder)

DXN = [-1 1 0; -1 0 1];DXF = DXN * X;T = abs(det(DXF))/2;INVDXF= inv(DXF);

[nq,w,p] = buildquadrature(quadorder,2);

sizephi = 3; if(polyorder==2) sizephi = 6;end

PhQ(1:nq,1:sizephi)=0;dxPhQ(1:nq,1:sizephi)=0;dyPhQ(1:nq,1:sizephi)=0;
PN1 =inv([1 0 0; 1 1 0; 1 0 1]);
weight = w * 2 * T;
XYQ = [ones(nq,1)  p] * PN1 * X;

if(polyorder == 1)
    PN =inv([1 0 0; 1 1 0; 1 0 1]);
    PhQ   = [ones(nq,1)  p] * PN;
    dxPhQ = [ones(nq,1) zeros(nq,1)] * INVDXF*[ 0 1 0; 0 0 1]*PN;
    dyPhQ = [zeros(nq,1) ones(nq,1)] * INVDXF*[ 0 1 0; 0 0 1]*PN;

elseif(polyorder == 2)

    Xhat=[[0 0];[1 0];[0 1];[0.5 0.5];[0 0.5];[0.5 0]];
    PN = inv([ones(6,1) Xhat Xhat(:,1).*Xhat(:,2) Xhat(:,1).^2 Xhat(:,2).^2]);
    PhQ = [ones(nq,1) p p(:,1).*p(:,2) p(:,1).^2 p(:,2).^2] * PN;
    for l = 1:nq
        xl = p(l,1); yl = p(l,2);
        dxPhQ(l,:) = [1 0] * INVDXF*[ 0 1 0 yl 2*xl 0; 0 0 1 xl 0 2*yl]*PN;
        dyPhQ(l,:) = [0 1] * INVDXF*[ 0 1 0 yl 2*xl 0; 0 0 1 xl 0 2*yl]*PN;
    end

else error('polynomial order out of range ');
end
