%% This function builds the degree of freedom for Pk-Lagrange fem 
%%
%% INPUTS  : 
%%           T  :  mesh 
%%           order : order of local polynomial 
%%
%% OUTPUTS : nbdof   : total degrees of freedom (dof)
%%           nbloc   : total local degree of freedom
%%           l2g     : array nt x nloc linking local number to global number
%%           doflab  : array nbdof x 1 of label of dof
%%           coord   : array nbdof x 2 coordinates of dof
%%
%%                  3
%%                 / \
%%                /   \
%%               /     \          order = 1
%%            1 /_______\ 2
%%
%%
%%
%%                  3
%%                 / \
%%              5 /   \4         order = 2
%%               /     \
%%            1 /_______\ 2
%%                 6
%%
%%
%%(c) Jean-Baptiste Apoung Kamga (July 2007)

function [l2g, nbloc, nbdof , doflab, coord] = builddof(T, order)

l2g=[];
%nbloc=[];
%nbdof=[];
doflab=[];
coord=[];

nt = size(T.Triangles,1);
nv = size(T.Vertices,1);
ne = size(T.e2t,1);

x=T.Vertices(:,1);
y=T.Vertices(:,2);

if (order == 0)
    nbloc = 1;
    nbdof = nt;
    l2g = (1:nt)';
    doflab = T.Tlabel;
    coord(:,1)  = (x(T.Triangles(:,1)) + x(T.Triangles(:,2)) + x(T.Triangles(:,3)))/3.;
    coord(:,2)  = (y(T.Triangles(:,1)) + y(T.Triangles(:,2)) + y(T.Triangles(:,3)))/3.;
elseif( order == 1)
    nbloc = 3;
    nbdof = nv;
    l2g = T.Triangles;
    doflab = T.Vlabel;
    coord(:,1)  = x;
    coord(:,2)  = y;
elseif( order == 2)
    nbloc = 6;
    %nbdof = nv + ne ;

    vdl(1:nv) = 0;
    edl(1:ne) = 0;

    xe = (x(T.e2v(:,1)) + x(T.e2v(:,2)))/2.;
    ye = (y(T.e2v(:,1)) + y(T.e2v(:,2)))/2.;

    nbdof= 0;

    for k=1:nt
        %working on vertices of the current triangle
        for i= 1:3
            il = T.Triangles(k,i);
            if( vdl(il) ~= 0)
                l2g(k,i) = vdl(il);
            else
                nbdof = nbdof + 1 ;
                l2g(k,i) = nbdof;
                vdl(il)  = nbdof;
                coord(nbdof,1) = x(il);
                coord(nbdof,2) = y(il);
                doflab(nbdof) = T.Vlabel(il);
            end
        end
        %working on edges of the current triangle
        for i = 1:3
            il = T.t2e(k,i);
            if( edl(il) ~= 0)
                l2g(k,i+3) = edl(il);
            else
                nbdof = nbdof + 1 ;
                l2g(k,i+3) = nbdof;
                edl(il)  = nbdof;
                coord(nbdof,1) = xe(il);
                coord(nbdof,2) = ye(il);
                doflab(nbdof) = T.elab(il);
            end
        end
    end
else
    error(' Undefined order');
end

%Verify
%  plot2dmesh(T,[1 0 0]);
%  hold on;
%  plot(coord(:,1),coord(:,2),'*b');
