%% This function builds the edge of the mesh 
%% and set connectivity
%%
%% INPUTS  : 
%%           T : the mesh
%%
%% OUTPUTS : T : the mesh with    new arrays
%%           t2t : array nt x 3 triangles to triangles connectivity   
%%           e2t : array ne x 2 edges to triangle connectivity e2t(k,0) is the left triangle
%%           e2v : array ne x 2 edges to vertices connectivity
%%           t2e : array nt x 3 triangles to edges connectivity
%%           v2e : array nv x nem edges containing the vertex
%%           v2t : array nv x ntm triangles containing the vertex
%%           elab: array ne x 1 label of edges
%% (c) Jean-Baptiste Apoung Kamga (July 2007)


function T=buildedges(T)

[nv,dim] = size(T.Vertices);
[nt,dim] = size(T.Triangles);
[nbe,dim] = size(T.Edges);

edgt = [ 2 3; 3 1 ; 1 2];
ne = 0;
head(1:nv)=0;
t2t(1:nt,1:3)=0;
%nedgesmax = nt + nv ;

%next = sparse(nedgesmax,1);

for k = 1:nt
for el = 1:3
i = T.Triangles(k,edgt(el,1)); 
j = T.Triangles(k,edgt(el,2));
% make i < j so that [i j] is the edge
d = 0; if(i > j) d=i; i = j; j=d; d =1;end
%check if this edge is stored
%look in all the edges having vertex i
e = head(i) ;
if(e == 0)
%this edge is not stored
ne = ne + 1;
e2v(ne, 1) = i; 
e2v(ne, 2) = j;
t2e(k,el) = ne;
e2t(ne, d + 1) = k;
%e2t(ne, d + 2) = el;
next(ne) = head(i);
head(i) = ne;
else
%there are edges passing by vertex i
% let us walk trought them 
found = 0 ;
while ( e ~= 0)
   if(e2v(e,2) == j)
   %this is the second time we are encounting this edge
     t2e(k,el) = e;
     e2t(e, d + 1) = k;
 %    e2t(e, d + 2) = el;
     found = 1;
   end
   e = next(e);
   end
if(found == 0)
%here the egde is not found
% so we add it
ne = ne + 1;
e2v(ne, 1) = i; 
e2v(ne, 2) = j;
t2e(k,el) = ne;
e2t(ne, d + 1) = k;
%e2t(ne, d + 2) = el;
next(ne) = head(i);
head(i) = ne;
end
end
end
end

%store vertex to edge connectivity
%for i=1:nv
%T.v2e(i,:) = next(i,find(next(i,:) ~= 0));
%endfor

%  t2t(e2t(:,1),1) = e2t(:,2)
%  t2t(e2t(:,1),1) = e2t(:,2)
%  
%  eint=e2t(:,3:4);
%  for k = 1:max(size(eint))
%  t2t(e2t(k,1), eint(k, 
%  endfor

for k = 1:nt
for i = 1:3
e = t2e(k,i);
if(e2t(e,1) == k) T.t2t(k,i) = e2t(e,2); end
if(e2t(e,2) == k) T.t2t(k,i) = e2t(e,1); end
end
end

%setting the label of the edges 
elab(1:ne)=0;
for k = 1:nbe
i = T.Edges(k,1); 
j = T.Edges(k,2); 
d = 0; if(i > j) d=i; i = j; j=d; d =1;end  
e = head(i);
while ( e ~= 0)
   if(e2v(e,2) == j)
     elab(e) = T.Elabel(k); 
   end
   e = next(e);
   end
   end

%storing
T.e2t = e2t;
T.t2e = t2e;
T.e2v = e2v;
T.elab = elab;

%T.head = head;
%T.next = next;

end