%% This function builds the quadrature formula of the  
%% reference triangle 
%%
%% INPUTS  : 
%%           order : order of quad formula
%%           dim   : 1 or 2 for edge or triangle
%%
%% OUTPUTS :  nq: number of quadrature points
%%            W : array nq x 1 of quadrature weights    
%%            P : array nq x (1 or 2) of quadrature coordinates 
%%
%% (c) Jean-Baptiste Apoung Kamga (July 2007)

function [nq,w,p] = buildquadrature(order,dim)

if(dim == 1)
    if(order == 1)
        nq = 1;
        w =[1];
        p =[0.5];
    elseif(order == 2)

    elseif(order == 3)
        nq = 2;
        a=0.5;
        x0=0.5-0.5/sqrt(3.);
        x1=0.5+0.5/sqrt(3.);
        w =[a a];
        p =[x0 ; x1];
    elseif(order == 4 || order == 5)
        nq = 3;
        a=5./18.;
        b= 4/8;
        x0=0.5;
        x1 = 0.5-0.5*sqrt(3./5.);
        x2 = 0.5+0.5*sqrt(3./5.);
        w =[b a a];
        p =[x0 ; x1; x2];
    else
        error('order out of range');
    end
elseif(dim == 2)

    if(order == 1)
        nq = 1;
        w = [1/2];
        p=[1/3,1/3];
    elseif(order == 2)
        nq = 3;
        w = [1./6. 1./6.  1/6. ];
        p = [[0.5, 0.]; [0.5, 0.5]; [0., 0.5] ];
    elseif(order == 3)
        nq = 4;
        w = [-27/96 25/96 25/96 25/96];
        p = [[1/3,1/3]; [1/5,1/5]; [3/5,1/5]; [1/5,3/5]];
    elseif(order == 4)
        nq = 6;
        a0=0.445948490915965;b0=0.091576213509771;
        area = 0.5;
        A=0.111690794839005*2*area;B=0.054975871827661*2*area;
        w = [A A A B B B];
        p = [ [a0,a0]; [1-2*a0, a0]; [a0, 1-2*a0] ; [b0,b0] ; [1-2*b0,b0] ; [b0,1-2*b0] ];
    elseif(order == 5 || order == 6)
        nq = 12;
        A=0.025422453185103;
        B=0.058393137863189;
        C=0.041425537809187;
        a0=0.063089014491502;b0=0.249286745170910;
        c0=0.310352451033785;d0=0.053145049844816;
        a10 = 1.-2.*a0; b10 = 1.-2.*b0; c10 = 1-c0-d0;
        w= [ A A A B B B C C C C C C];
        p = [[a0,a0]; [a10,a0]; [a0,a10]; [b0,b0];[b10,b0]; [b0,b10]; [c0,d0];[d0,c0];[c10,c0];[c10,d0]; [c0,c10]; [d0,c10]];

    else
        error('order out of range');
    end
else
    error('dimension out of range');
end
